#include "search.h"
#include <linux/perf_event.h> 
#include <inttypes.h>
#include <sys/ioctl.h>
#include <sys/syscall.h> 
#include <x86intrin.h> // rdtsc

#define TOTAL_EVENTS 6

// Executes perf_event_open syscall and makes sure it is successful or exit
static long perf_event_open(struct perf_event_attr *hw_event, pid_t pid, int cpu, int group_fd, unsigned long flags){
  int fd;
  fd = syscall(SYS_perf_event_open, hw_event, pid, cpu, group_fd, flags);
  if (fd == -1) {
    fprintf(stderr, "Error creating event");
    exit(EXIT_FAILURE);
  }

  return fd;
}

void configure_event(struct perf_event_attr *pe, uint32_t type, uint64_t config){
  memset(pe, 0, sizeof(struct perf_event_attr));
  pe->type = type;
  pe->size = sizeof(struct perf_event_attr);
  pe->config = config;
  pe->read_format = PERF_FORMAT_GROUP | PERF_FORMAT_ID;
  pe->disabled = 1;
  pe->exclude_kernel = 1;
  pe->exclude_hv = 1;
}

int main(int argc, char **argv){
    int fd_ic;
    uint64_t  val_ic;
    struct perf_event_attr pe_ic;
    memset(&pe_ic, 0, sizeof(struct perf_event_attr));
    pe_ic.type = PERF_TYPE_HARDWARE;
    pe_ic.size = sizeof(struct perf_event_attr);
    pe_ic.config = PERF_COUNT_HW_INSTRUCTIONS;
    pe_ic.disabled = 1;
    pe_ic.exclude_kernel = 1;   
    pe_ic.exclude_hv = 1;  
    
    fd_ic = perf_event_open(&pe_ic, 0, -1, -1, 0);


    struct timespec t1, t2;
    char * buffer = generate_input();
    printf("Buffer of size %d Gb loaded \n",  buffer_size/GB);
    clock_gettime(CLOCK_MONOTONIC, &t1);
    ioctl(fd_ic, PERF_EVENT_IOC_RESET, 0);
    ioctl(fd_ic, PERF_EVENT_IOC_ENABLE, 0);
    uint64_t cycle_in = __rdtsc();
    char * result = search(buffer, buffer_size);
    uint64_t cycle_out = __rdtsc();
    read(fd_ic, &val_ic, sizeof(val_ic));  
    clock_gettime(CLOCK_MONOTONIC, &t2);
    double time = ((float) format_elapsed_time(t1, t2))/NANOSEC_CONVERSION;
    printf("Time:%fs\n", time);
    printf("Instruction/o:%.2f\n", ((double) val_ic) / buffer_size);
    printf("Cycle/o:%.2f\n", ((double) (cycle_out - cycle_in)) / buffer_size);
    printf("GB/s:%.2f\n", (((double) buffer_size)/GB)/time);
    printf("position: %lu\n", result !=NULL? result - buffer:0);
    free(buffer);
    return 0;
}
